@@ -21,6 +21,16 @@ sub new {
 }
 
 
+my $window_function = 'window_function';
+my $args_str = 'args_str';
+my $partition_by_stmt = 'partition_by_stmt';
+my $order_by_stmt = 'order_by_stmt';
+my $frame_mode = 'frame_mode';
+my $frame_start = 'frame_start';
+my $frame_end = 'frame_end';
+my $frame_exclusion = 'frame_exclusion';
+
+
 sub __choose_a_column {
     my ( $sf, $sql, $clause, $cols, $r_data ) = @_;
     my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
@@ -45,7 +55,7 @@ sub __choose_a_column {
             # from 'window_function': to avoid window function in window function
             my $complex_col = $ext->column(
                 $sql, $clause, $r_data,
-                { from => 'window_function' }
+                { from => $window_function }
             );
             if ( ! defined $complex_col ) {
                 next;
@@ -62,61 +72,61 @@ sub __get_win_func_stmt {
     $placeholder //= '';
     my @parts;
     my $win_stmt = $win_data->{func};
-    $win_stmt .= '(' . $win_data->{args_str} // '';
-    if ( $placeholder eq 'args_str' ) {
+    $win_stmt .= '(' . $win_data->{$args_str} // '';
+    if ( $placeholder eq $args_str ) {
         push @parts, $win_stmt;
         $win_stmt = '';
     }
     $win_stmt .= ') OVER (';
-    if ( length $win_data->{partition_by_stmt} ) {
-        $win_stmt .= $win_data->{partition_by_stmt} . " ";
+    if ( length $win_data->{$partition_by_stmt} ) {
+        $win_stmt .= $win_data->{$partition_by_stmt} . " ";
     }
-    if ( $placeholder eq 'partition_by_stmt' ) {
+    if ( $placeholder eq $partition_by_stmt ) {
         push @parts, $win_stmt;
         $win_stmt = '';
     }
-    if ( length $win_data->{order_by_stmt} ) {
-        $win_stmt .= $win_data->{order_by_stmt} . " ";
+    if ( length $win_data->{$order_by_stmt} ) {
+        $win_stmt .= $win_data->{$order_by_stmt} . " ";
     }
-    if ( $placeholder eq 'order_by_stmt' ) {
+    if ( $placeholder eq $order_by_stmt ) {
         push @parts, $win_stmt;
         $win_stmt = '';
     }
-    if ( length $win_data->{frame_mode} ) {
-        $win_stmt .= $win_data->{frame_mode} . " ";
-        if ( length $win_data->{frame_start} || $placeholder eq 'frame_start' ) {
-            if ( length $win_data->{frame_end} || $placeholder eq 'frame_end' ) {
+    if ( length $win_data->{$frame_mode} ) {
+        $win_stmt .= $win_data->{$frame_mode} . " ";
+        if ( length $win_data->{$frame_start} || $placeholder eq $frame_start ) {
+            if ( length $win_data->{$frame_end} || $placeholder eq $frame_end ) {
                 $win_stmt .= "BETWEEN ";
             }
-            if ( length $win_data->{frame_start} ) {
-                $win_stmt .= $win_data->{frame_start} . " ";
+            if ( length $win_data->{$frame_start} ) {
+                $win_stmt .= $win_data->{$frame_start} . " ";
             }
-            if ( $placeholder eq 'frame_start' ) {
+            if ( $placeholder eq $frame_start ) {
                 push @parts, $win_stmt;
                 $win_stmt = '';
             }
         }
-        if ( length $win_data->{frame_end} || $placeholder eq 'frame_end' ) {
-            if ( length $win_data->{frame_start} || $placeholder eq 'frame_start' ) {
+        if ( length $win_data->{$frame_end} || $placeholder eq $frame_end ) {
+            if ( length $win_data->{$frame_start} || $placeholder eq $frame_start ) {
                 $win_stmt .= "AND ";
             }
-            if ( length $win_data->{frame_end} ) {
-                $win_stmt .= $win_data->{frame_end} . " ";
+            if ( length $win_data->{$frame_end} ) {
+                $win_stmt .= $win_data->{$frame_end} . " ";
             }
-            if ( $placeholder eq 'frame_end' ) {
+            if ( $placeholder eq $frame_end ) {
                 push @parts, $win_stmt;
                 $win_stmt = '';
             }
         }
-        if ( length $win_data->{frame_exclusion} ) {
-            $win_stmt .= $win_data->{frame_exclusion};
+        if ( length $win_data->{$frame_exclusion} ) {
+            $win_stmt .= $win_data->{$frame_exclusion};
         }
-        if ( $placeholder eq 'frame_exclusion' ) {
+        if ( $placeholder eq $frame_exclusion ) {
             push @parts, $win_stmt;
             $win_stmt = '';
         }
     }
-    if ( $placeholder eq 'frame_mode' ) {
+    if ( $placeholder eq $frame_mode ) {
         push @parts, $win_stmt;
         $win_stmt = '';
     }
@@ -154,7 +164,7 @@ sub window_function {
     my @func_with_offset = ( 'LAG', 'LEAD', 'NTH_VALUE' );
     my $rx_func_with_offset = join( '|', map { quotemeta } @func_with_offset );
 
-    my @func_with_offset_and_default = ( 'LAG', 'LEAD' );
+    my @func_with_offset_and_default = ( 'LAG', 'LEAD' ); ##
     my $rx_func_with_offset_and_default = join( '|', map { quotemeta } @func_with_offset_and_default );
 
     my $info_sql = $ax->get_sql_info( $sql );
@@ -193,8 +203,8 @@ sub window_function {
         $win_data->{func} = $func;
 
         COLUMN: while ( 1 ) {
-            $win_data->{args_str} = '';
-            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, 'args_str' ) ];
+            $win_data->{$args_str} = '';
+            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, $args_str ) ];
             my $col;
             if ( $func =~ /^$rx_func_count_all\z/i ) {
                 $col = '*';
@@ -207,7 +217,7 @@ sub window_function {
                 # Readline
                 $col = $ext->argument(
                     $sql, $clause, $r_data,
-                    { history => undef, prompt => 'n = ', is_numeric => 1, from => 'window_function' }
+                    { history => undef, prompt => 'n = ', is_numeric => 1, from => $window_function }
                 );
                 if ( ! length $col || $col eq "''" ) {
                     next WINDOW_FUNCTION;
@@ -219,26 +229,26 @@ sub window_function {
                     next WINDOW_FUNCTION;
                 }
             }
-            $win_data->{args_str} = $col;
+            $win_data->{$args_str} = $col;
             if ( $func =~ /^(?:$rx_func_with_offset)\z/i ) {
-                $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, 'args_str' ) ];
+                $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, $args_str ) ];
                 # Readline
                 my $offset = $ext->argument(
                     $sql, $clause, $r_data,
                     { prompt => 'offset: ', is_numeric => 1 }
                 );
                 if ( length $offset && $offset ne "''" ) {
-                    $win_data->{args_str} .= ',' . $offset;
+                    $win_data->{$args_str} .= ',' . $offset;
                     if ( $func =~ /^(?:$rx_func_with_offset_and_default)\z/i ) {
-                        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, 'args_str' ) ];
+                        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, $args_str ) ];
                         my $is_numeric = $ax->is_numeric( $sql, $col );
                         # Readline
                         my $default_value = $ext->argument(
                             $sql, $clause, $r_data,
-                            { prompt => 'default: ', is_numeric => $is_numeric, from => 'window_function' }
+                            { prompt => 'default: ', is_numeric => $is_numeric, from => $window_function }
                         );
                         if ( length $default_value && $default_value ne "''" ) {
-                            $win_data->{args_str} .= ',' . $default_value;
+                            $win_data->{$args_str} .= ',' . $default_value;
                             #$r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data ) ];
                         }
                     }
@@ -259,8 +269,8 @@ sub window_function {
                       undef => $sf->{i}{back} }
                 );
                 if ( ! defined $idx_wd || ! defined $menu->[$idx_wd] ) {
-                    if ( $win_data->{partition_by_stmt} || $win_data->{order_by_stmt} || $win_data->{frame_mode} ) {
-                        $win_data = { func => $win_data->{func}, args_str => $win_data->{args_str} };
+                    if ( $win_data->{$partition_by_stmt} || $win_data->{$order_by_stmt} || $win_data->{$frame_mode} ) {
+                        $win_data = { func => $win_data->{func}, args_str => $win_data->{$args_str} };
                         next WINDOW_DEFINITION;
                     }
                     if ( $func =~ /^(?:$rx_func_count_all|$rx_func_no_col)\z/ ) {
@@ -311,12 +321,12 @@ sub __add_partition_by {
 
     PARTITION_BY: while ( 1 ) {
         if ( @{$win_data->{partition_by_cols}} ) {
-            $win_data->{partition_by_stmt} = "PARTITION BY " . join ',', @{$win_data->{partition_by_cols}};
+            $win_data->{$partition_by_stmt} = "PARTITION BY " . join ',', @{$win_data->{partition_by_cols}};
         }
         else {
-            $win_data->{partition_by_stmt} = "PARTITION BY";
+            $win_data->{$partition_by_stmt} = "PARTITION BY";
         }
-        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, 'partition_by_stmt' ) ];
+        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, $partition_by_stmt ) ];
         my $info = $info_sql . $ext->nested_func_info( $r_data );
         my $prompt = 'Partition by:';
         # Choose
@@ -331,17 +341,17 @@ sub __add_partition_by {
                 pop @{$win_data->{partition_by_cols}};
                 next PARTITION_BY;
             }
-            delete $win_data->{partition_by_stmt};
+            delete $win_data->{$partition_by_stmt};
             return;
         }
         elsif ( $menu->[$idx[0]] eq $sf->{i}{ok} ) {
             shift @idx;
             push @{$win_data->{partition_by_cols}}, @{$menu}[@idx];
             if ( ! @{$win_data->{partition_by_cols}} ) {
-                delete $win_data->{partition_by_stmt};
+                delete $win_data->{$partition_by_stmt};
             }
             else {
-                $win_data->{partition_by_stmt} = "PARTITION BY " . join ',', @{$win_data->{partition_by_cols}};
+                $win_data->{$partition_by_stmt} = "PARTITION BY " . join ',', @{$win_data->{partition_by_cols}};
             }
             return 1;
         }
@@ -349,7 +359,7 @@ sub __add_partition_by {
             my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
             my $complex_column = $ext->column(
                 $sql, $clause, $r_data,
-                { from => 'window_function' }
+                { from => $window_function }
             );
             if ( defined $complex_column ) {
                 push @{$win_data->{partition_by_cols}}, $complex_column;
@@ -375,12 +385,12 @@ sub __add_order_by {
 
     ORDER_BY: while ( 1 ) {
         if ( @{$win_data->{order_by_cols}} ) {
-            $win_data->{order_by_stmt} = "ORDER BY " . join ',', @{$win_data->{order_by_cols}};
+            $win_data->{$order_by_stmt} = "ORDER BY " . join ',', @{$win_data->{order_by_cols}};
         }
         else {
-            $win_data->{order_by_stmt} = "ORDER BY";
+            $win_data->{$order_by_stmt} = "ORDER BY";
         }
-        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, 'order_by_stmt' ) ];
+        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, $order_by_stmt ) ];
         my $info = $info_sql . $ext->nested_func_info( $r_data );
         my $prompt = 'Order by:';
         # Choose
@@ -394,15 +404,15 @@ sub __add_order_by {
                 pop @{$win_data->{order_by_cols}};
                 next ORDER_BY;
             }
-            delete $win_data->{order_by_stmt};
+            delete $win_data->{$order_by_stmt};
             return
         }
         if ( $col eq $sf->{i}{ok} ) {
             if ( ! @{$win_data->{order_by_cols}} ) {
-                delete $win_data->{order_by_stmt};
+                delete $win_data->{$order_by_stmt};
             }
             else {
-                $win_data->{order_by_stmt} = "ORDER BY " . join ',', @{$win_data->{order_by_cols}};
+                $win_data->{$order_by_stmt} = "ORDER BY " . join ',', @{$win_data->{order_by_cols}};
             }
             return 1;
         }
@@ -410,7 +420,7 @@ sub __add_order_by {
             my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
             my $complex_column = $ext->column(
                 $sql, $clause, $r_data,
-                { from => 'window_function' }
+                { from => $window_function }
             );
             if ( ! defined $complex_column ) {
                 next ORDER_BY;
@@ -418,8 +428,8 @@ sub __add_order_by {
             $col = $complex_column;
         }
         push @{$win_data->{order_by_cols}}, $col;
-        $win_data->{order_by_stmt} = "ORDER BY " . join ',', @{$win_data->{order_by_cols}};
-        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, 'order_by_stmt' ) ];
+        $win_data->{$order_by_stmt} = "ORDER BY " . join ',', @{$win_data->{order_by_cols}};
+        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, $order_by_stmt ) ];
         $info = $info_sql . $ext->nested_func_info( $r_data );
         # Choose
         my $direction = $tc->choose(
@@ -447,8 +457,8 @@ sub __add_frame_clause {
     if ( $sf->{i}{driver} =~ /^(?:SQLite|Pg|Oracle)\z/ ) {
         push @frame_clause_modes, 'GROUPS';
     }
-    if ( ! length $win_data->{frame_mode} ) {
-        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, 'frame_mode' ) ];
+    if ( ! length $win_data->{$frame_mode} ) {
+        $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, $frame_mode ) ];
     }
     my $info_sql = $ax->get_sql_info( $sql );
     my $old_idx_fc = 0;
@@ -474,7 +484,7 @@ sub __add_frame_clause {
             $old_idx_fc = $idx_fc;
         }
         my $frame_mode = $frame_clause_modes[$idx_fc-@pre];
-        $win_data->{frame_mode} = $frame_mode;
+        $win_data->{$frame_mode} = $frame_mode;
         my $old_idx_fe = 0;
 
         FRAME_END_AND_EXCLUSION: while ( 1 ) {
@@ -486,7 +496,7 @@ sub __add_frame_clause {
             if ( $sf->{i}{driver} =~ /^(?:SQLite|Pg|Oracle)\z/ ) {
                 push @$menu, $frame_exclusion;
             }
-            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, 'frame_mode' ) ];
+            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, $frame_mode ) ];
             my $info = $info_sql . $ext->nested_func_info( $r_data );
             # Choose
             my $idx_fe = $tc->choose(
@@ -495,11 +505,11 @@ sub __add_frame_clause {
             );
             $ax->print_sql_info( $info );
             if ( ! defined $idx_fe || ! defined $menu->[$idx_fe] ) {
-                if ( $win_data->{frame_start} || $win_data->{frame_end} || $win_data->{frame_exclusion} ) {
+                if ( $win_data->{$frame_start} || $win_data->{$frame_end} || $win_data->{$frame_exclusion} ) {
                     delete @{$win_data}{qw(frame_start frame_end frame_exclusion)};
                     next FRAME_END_AND_EXCLUSION;
                 }
-                delete $win_data->{frame_mode};
+                delete $win_data->{$frame_mode};
                 next FRAME_CLAUSE;
             }
             if ( $sf->{o}{G}{menu_memory} ) {
@@ -514,10 +524,10 @@ sub __add_frame_clause {
                 return 1;
             }
             if ( $choice eq $frame_start ) {
-                $sf->__add_frame_start_or_end( $sql, $clause, $r_data, $win_data, 'frame_start' );
+                $sf->__add_frame_start_or_end( $sql, $clause, $r_data, $win_data, $frame_start );
             }
             elsif ( $choice eq $frame_end ) {
-                $sf->__add_frame_start_or_end( $sql, $clause, $r_data, $win_data, 'frame_end' );
+                $sf->__add_frame_start_or_end( $sql, $clause, $r_data, $win_data, $frame_end );
             }
             else {
                 $sf->__add_frame_exclusion( $sql, $r_data, $win_data );
@@ -533,11 +543,11 @@ sub __add_frame_start_or_end {
     my $ext = App::DBBrowser::Table::Extensions->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my ( @frame_point_types, $prompt );
-    if ( $pos eq 'frame_start' ) {
+    if ( $pos eq $frame_start ) {
         @frame_point_types = ( 'UNBOUNDED PRECEDING', 'n PRECEDING', 'CURRENT ROW', 'n FOLLOWING' );
         $prompt = 'Frame start:';
     }
-    elsif ( $pos eq 'frame_end' ) {
+    elsif ( $pos eq $frame_end ) {
         @frame_point_types = ( 'n PRECEDING', 'CURRENT ROW', 'n FOLLOWING', 'UNBOUNDED FOLLOWING' );
         $prompt = 'Frame end:';
     }
@@ -598,8 +608,8 @@ sub __add_frame_exclusion {
     my $info_sql = $ax->get_sql_info( $sql );
 
     FRAME_EXCLUSION: while ( 1 ) {
-        if ( ! length $win_data->{frame_exclusion} ) {
-            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, 'frame_exclusion' ) ];
+        if ( ! length $win_data->{$frame_exclusion} ) {
+            $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data, $frame_exclusion ) ];
         }
         else {
             $r_data->[-1] = [ 'win', $sf->__get_win_func_stmt( $win_data ) ];
@@ -615,7 +625,7 @@ sub __add_frame_exclusion {
         );
         $ax->print_sql_info( $info );
         if ( ! defined $frame_exclusion ) {
-            delete $win_data->{frame_exclusion};
+            delete $win_data->{$frame_exclusion};
             return;
         }
         elsif ( $frame_exclusion eq $confirm ) {
@@ -623,7 +633,7 @@ sub __add_frame_exclusion {
         }
         else {
             $frame_exclusion =~ s/^-\s//;
-            $win_data->{frame_exclusion} = $frame_exclusion;
+            $win_data->{$frame_exclusion} = $frame_exclusion;
         }
     }
 }
