@@ -3,7 +3,7 @@ App::DBBrowser::Table::Substatement::Aggregate;
 
 use warnings;
 use strict;
-use 5.016;
+use 5.014;
 
 use Term::Choose         qw();
 use Term::Form::ReadLine qw();
@@ -29,7 +29,7 @@ sub __group_concat {
     if ( $driver =~ /^(?:SQLite|mysql|MariaDB)\z/ ) {
         $group_concat = "GROUP_CONCAT";
     }
-    elsif ( $driver =~ /^(?:Pg|DuckDB)\z/ ) {
+    elsif ( $driver eq 'Pg' ) {
         $group_concat = "STRING_AGG";
     }
     elsif ( $driver eq 'Firebird' ) {
@@ -69,25 +69,7 @@ sub get_prepared_aggr_func {
         if ( $sf->{o}{enable}{extended_cols} ) {
             push @pre, $sf->{i}{menu_addition};
         }
-        my $group_concat = $sf->__group_concat();
         $prepared_aggr = $aggr . "(";
-        if ( $aggr =~ /^(?:COUNT|$group_concat)\z/i ) {
-            my ( $all, $distinct ) = ( 'ALL', 'DISTINCT' );
-            my $info = $sf->__prepared_aggr_info( $sql, $clause, $prepared_aggr, $r_data );
-            # Choose
-            my $choice = $tc->choose(
-                [ undef, $all, $distinct ],
-                { %{$sf->{i}{lyt_h}}, info => $info }
-            );
-            $ax->print_sql_info( $info );
-            if ( ! defined $choice ) {
-                return;
-            }
-            #if ( $choice eq $distinct ) {
-            #    $prepared_aggr .= "DISTINCT ";
-            #}
-            $prepared_aggr .= $choice . " ";
-        }
 
         COLUMN: while ( 1 ) {
             my $info = $sf->__prepared_aggr_info( $sql, $clause, $prepared_aggr, $r_data );
@@ -113,12 +95,22 @@ sub get_prepared_aggr_func {
                 }
                 $col = $complex_col;
             }
-            if ( $prepared_aggr =~ /ALL\s\z/ ) {
-                $prepared_aggr = $aggr . "(";
-            };
-            if ( $aggr =~ /^$group_concat\z/i ) {
+            my $group_concat = $sf->__group_concat();
+            if ( $aggr =~ /^COUNT\z/i ) {
+                my $is_distinct = $sf->__is_distinct( $sql, $clause, $prepared_aggr . $col, $r_data );
+                if ( ! defined $is_distinct ) {
+                    next COLUMN;
+                }
+                if ( $is_distinct ) {
+                    $prepared_aggr .= "DISTINCT $col)";
+                }
+                else {
+                    $prepared_aggr .= "$col)";
+                }
+            }
+            elsif ( $aggr =~ /^$group_concat\z/i ) {
                 my $bu_prepared_aggr = $prepared_aggr;
-                $prepared_aggr = $sf->__op_group_concat( $sql, $clause, $col, $prepared_aggr, $r_data );
+                $prepared_aggr = $sf->__opt_group_concat( $sql, $clause, $col, $prepared_aggr, $r_data );
                 if ( ! defined $prepared_aggr ) {
                     $prepared_aggr = $bu_prepared_aggr;
                     next COLUMN;
@@ -136,6 +128,29 @@ sub get_prepared_aggr_func {
 }
 
 
+sub __is_distinct {
+    my ( $sf, $sql, $clause, $prepared_aggr, $r_data ) = @_;
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $tc = Term::Choose->new( $sf->{i}{tc_default} );
+    my ( $all, $distinct ) = ( 'ALL', 'DISTINCT' );
+    my $info = $sf->__prepared_aggr_info( $sql, $clause, $prepared_aggr, $r_data );
+    # Choose
+    my $choice = $tc->choose(
+        [ undef, $all, $distinct ],
+        { %{$sf->{i}{lyt_h}}, info => $info }
+    );
+    $ax->print_sql_info( $info );
+    if ( ! defined $choice ) {
+        return;
+    }
+    elsif ( $choice eq $all ) {
+        return 0;
+    }
+    elsif ( $choice eq $distinct ) {
+        return 1;
+    }
+}
+
 sub __prepared_aggr_info {
     my ( $sf, $sql, $clause, $prepared_aggr, $r_data ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
@@ -146,13 +161,18 @@ sub __prepared_aggr_info {
 }
 
 
-sub __op_group_concat {
+sub __opt_group_concat {
     my ( $sf, $sql, $clause, $col, $prepared_aggr, $r_data ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
-    my $driver = $sf->{i}{driver};
-    my $is_distinct = $prepared_aggr =~ /DISTINCT\s\z/;
-    if ( $driver eq 'Pg' ) {
+    my $is_distinct = $sf->__is_distinct( $sql, $clause, $prepared_aggr . $col, $r_data );
+    if ( ! defined $is_distinct ) {
+        return;
+    }
+    if ( $is_distinct ) {
+        $prepared_aggr .= "DISTINCT ";
+    }
+    if ( $sf->{i}{driver} eq 'Pg' ) {
         $prepared_aggr .= $ax->pg_column_to_text( $sql, $col );
     }
     else {
@@ -160,14 +180,18 @@ sub __op_group_concat {
     }
     my $sep = ',';
     my $order_by_stmt;
-    if (      $driver =~ /^(?:mysql|MariaDB|Pg|DuckDB)\z/
-         || ( $driver =~ /^(?:DB2|Oracle)\z/ && ! $is_distinct )
+    if (      $sf->{i}{driver} =~ /^(?:mysql|MariaDB|Pg)\z/
+         || ( $sf->{i}{driver} =~ /^(?:DB2|Oracle)\z/ && ! $is_distinct )
     ) {
         my $read = ':Read';
-        if ( $driver eq 'Pg' && $is_distinct ) {
+        if ( $sf->{i}{driver} eq 'Pg' && $is_distinct ) {
             $col = $ax->pg_column_to_text( $sql, $col );
         }
-        my @choices = ( "ASC", "DESC", $read );
+        my @choices = (
+            "$col ASC",
+            "$col DESC",
+            $read,
+        );
         my $menu = [ undef, @choices ];
         my $info = $sf->__prepared_aggr_info( $sql, $clause, $prepared_aggr, $r_data );
         # Choose
@@ -197,10 +221,10 @@ sub __op_group_concat {
             }
         }
         else {
-            $order_by_stmt = "ORDER BY $col $choice";
+            $order_by_stmt = "ORDER BY " . $choice;
         }
     }
-    if ( $driver eq 'SQLite' ) {
+    if ( $sf->{i}{driver} eq 'SQLite' ) {
         if ( $is_distinct ) {
             # https://sqlite.org/forum/info/221c2926f5e6f155
             # SQLite: GROUP_CONCAT with DISTINCT and custom seperator does not work
@@ -211,7 +235,7 @@ sub __op_group_concat {
             $prepared_aggr .= ",'$sep')";
         }
     }
-    elsif ( $driver =~ /^(?:mysql|MariaDB)\z/ ) {
+    elsif ( $sf->{i}{driver} =~ /^(?:mysql|MariaDB)\z/ ) {
         if ( $order_by_stmt ) {
             $prepared_aggr .= " $order_by_stmt SEPARATOR '$sep')";
         }
@@ -219,11 +243,11 @@ sub __op_group_concat {
             $prepared_aggr .= " SEPARATOR '$sep')";
         }
     }
-    elsif ( $driver =~ /^(?:Pg|DuckDB)\z/ ) {
+    elsif ( $sf->{i}{driver} eq 'Pg' ) {
         # Pg, STRING_AGG:
-        # - separator mandatory
-        # - expects text type as argument
-        # - with DISTINCT the STRING_AGG col and the ORDER BY col must be identical
+        # separator mandatory
+        # expects text type as argument
+        # with DISTINCT the STRING_AGG col and the ORDER BY col must be identical
         if ( $order_by_stmt ) {
             $prepared_aggr .= ",'$sep' $order_by_stmt)";
         }
@@ -231,10 +255,10 @@ sub __op_group_concat {
             $prepared_aggr .= ",'$sep')";
         }
     }
-    elsif ( $driver eq 'Firebird' ) {
+    elsif ( $sf->{i}{driver} eq 'Firebird' ) {
         $prepared_aggr .= ",'$sep')";
     }
-    elsif ( $driver =~ /^(?:DB2|Oracle)\z/ ) {
+    elsif ( $sf->{i}{driver} =~ /^(?:DB2|Oracle)\z/ ) {
         # No order with distinct
         # DB2 codes: error code -214 - error caused by:
         # DISTINCT is specified in the SELECT clause, and a column name or sort-key-expression in the
